/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2016
* 
*        (as is).
*     .
********************************************************************************
*/


#include "cloud_board.h"



static volatile uint8_t State = 0;
static volatile uint8_t UpdateValue = 10;
static volatile uint8_t UpdateCnt = 0;
static volatile uint8_t ThunderDur = 0;
static volatile uint8_t ActiveRainCol = 0;
static volatile uint16_t LightCnt = 0;
static volatile uint8_t RainPower = 1;
static volatile uint8_t CurPos[COLS_NUM];


int main()
{
  Cloud_Init();
  
  SCB->SCR = SCB_SCR_SLEEPONEXIT_Msk;
  
  for(;;)
  {
    __WFI();
  }
}
//------------------------------------------------------------------------------
void Cloud_Init()
{
  RCC->AHBENR |= RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOFEN;
  
  LEDS_GPIO->MODER &= ~((3 << (2 * ROW1_PIN)) | (3 << (2 * ROW2_PIN)) | (3 << (2 * ROW3_PIN)) | \
                        (3 << (2 * ROW4_PIN)) | (3 << (2 * ROW5_PIN)) | (3 << (2 * COL1_PIN)) | \
                        (3 << (2 * COL2_PIN)) | (3 << (2 * COL3_PIN)) | (3 << (2 * COL4_PIN)));
  
  LEDS_GPIO->MODER |=  ((1 << (2 * ROW1_PIN)) | (1 << (2 * ROW2_PIN)) | (1 << (2 * ROW3_PIN)) | \
                        (1 << (2 * ROW4_PIN)) | (1 << (2 * ROW5_PIN)) | (1 << (2 * COL1_PIN)) | \
                        (1 << (2 * COL2_PIN)) | (1 << (2 * COL3_PIN)) | (1 << (2 * COL4_PIN)));
  
  BUZZ_GPIO->MODER &= ~(3 << (2 * BUZZ_PIN));
  BUZZ_GPIO->MODER |=  (1 << (2 * BUZZ_PIN));
  
  // :  = 1/4 
  RCC->TIME_BASE_TIMER_ENR |= TIME_BASE_TIMER_CLK_EN;
  TIME_BASE_TIMER->PSC = 3999;
  TIME_BASE_TIMER->ARR = 499;
  TIME_BASE_TIMER->DIER = TIM_DIER_UIE;
  NVIC_EnableIRQ(TIME_BASE_TIMER_IRQ);
  NVIC_SetPriority(TIME_BASE_TIMER_IRQ, 0);
  
  //    :  = 5 
  RCC->RAIN_TIMER_ENR |= RAIN_TIMER_CLK_EN;
  RAIN_TIMER->PSC = 399;
  RAIN_TIMER->ARR = 99;
  RAIN_TIMER->DIER = TIM_DIER_UIE;
  NVIC_EnableIRQ(RAIN_TIMER_IRQ);
  NVIC_SetPriority(RAIN_TIMER_IRQ, 2);
  
  //  
  RCC->THUNDER_TIMER_ENR |= THUNDER_TIMER_CLK_EN;
  THUNDER_TIMER->PSC = 19;//399;
  THUNDER_TIMER->ARR = 299;//399;
  THUNDER_TIMER->DIER = TIM_DIER_UIE;
  NVIC_EnableIRQ(THUNDER_TIMER_IRQ);
  NVIC_SetPriority(THUNDER_TIMER_IRQ, 1);
  
  
  TIME_BASE_TIMER->CR1 |= TIM_CR1_CEN;
}
//------------------------------------------------------------------------------
void TIME_BASE_TIMER_ISR(void)
{
  static uint16_t TimeCnt = SCENE_1_TIME - 1;
  
  TIME_BASE_TIMER->SR &= ~TIM_SR_UIF;
  
  TimeCnt++;
  
  switch(TimeCnt)
  {    
    case SCENE_1_TIME:
    {
      ActiveRainCol = ACTIVE_RAIN_1;
      RainPower = 1;
      UpdateValue = 10;
      UpdateCnt = 0;
      State |= RAIN;
      
      uint8_t RAIN_INIT_POS[COLS_NUM] = {RAIN_INIT_POS_COL1, RAIN_INIT_POS_COL2, RAIN_INIT_POS_COL3, RAIN_INIT_POS_COL4};
      for (uint8_t i = 0; i < COLS_NUM; i++)
        CurPos[i] = RAIN_INIT_POS[i];
      
      RAIN_TIMER->CR1 |= TIM_CR1_CEN;
    }
    break;
    
    case (SCENE_1_TIME + SCENE_1_DURATION):
    case (SCENE_2_TIME + SCENE_2_DURATION):
    case (SCENE_3_TIME + SCENE_3_DURATION):
    case (SCENE_4_TIME + SCENE_4_DURATION):
    case (SCENE_5_TIME + SCENE_5_DURATION):
    case (SCENE_6_TIME + SCENE_6_DURATION):
    {
      RAIN_TIMER->CR1 &= ~TIM_CR1_CEN;
      LEDS_GPIO->ODR &= ~(COLS_MASK | ROWS_MASK);
      State = 0;
      ActiveRainCol = 0;
      if (TimeCnt == END_TIME)
        TimeCnt = 0;
    }
    break;
    
    case SCENE_2_TIME:
    {
      ActiveRainCol = ACTIVE_RAIN_3;
      RainPower = 1;
      UpdateValue = 10;
      UpdateCnt = 0;
      State |= RAIN;
      
      uint8_t RAIN_INIT_POS[COLS_NUM] = {RAIN_INIT_POS_COL1, RAIN_INIT_POS_COL2, RAIN_INIT_POS_COL3, RAIN_INIT_POS_COL4};
      for (uint8_t i = 0; i < COLS_NUM; i++)
        CurPos[i] = RAIN_INIT_POS[i];
      
      RAIN_TIMER->CR1 |= TIM_CR1_CEN;
    }
    break;
    
    case (SCENE_2_TIME + SCENE_2_DURATION +     (SCENE_3_PAUSE / 4)):
    case (SCENE_2_TIME + SCENE_2_DURATION + 2 * (SCENE_3_PAUSE / 4)):
    case (SCENE_2_TIME + SCENE_2_DURATION + 3 * (SCENE_3_PAUSE / 4)):
    case (SCENE_3_TIME + SCENE_3_DURATION +     (SCENE_4_PAUSE / 8)):
    case (SCENE_3_TIME + SCENE_3_DURATION + 2 * (SCENE_4_PAUSE / 8)):
    case (SCENE_3_TIME + SCENE_3_DURATION + 3 * (SCENE_4_PAUSE / 8)):
    case (SCENE_3_TIME + SCENE_3_DURATION + 4 * (SCENE_4_PAUSE / 8)):
    case (SCENE_3_TIME + SCENE_3_DURATION + 5 * (SCENE_4_PAUSE / 8)):
    case (SCENE_3_TIME + SCENE_3_DURATION + 6 * (SCENE_4_PAUSE / 8)):
    case (SCENE_3_TIME + SCENE_3_DURATION + 7 * (SCENE_4_PAUSE / 8)):
    case (SCENE_4_TIME + SCENE_4_DURATION +     (SCENE_5_PAUSE / 6)):
    case (SCENE_4_TIME + SCENE_4_DURATION + 2 * (SCENE_5_PAUSE / 6)):
    case (SCENE_4_TIME + SCENE_4_DURATION + 3 * (SCENE_5_PAUSE / 6)):
    case (SCENE_4_TIME + SCENE_4_DURATION + 4 * (SCENE_5_PAUSE / 6)):
    case (SCENE_4_TIME + SCENE_4_DURATION + 5 * (SCENE_5_PAUSE / 6)):
    case (SCENE_5_TIME + SCENE_5_DURATION +     (SCENE_6_PAUSE / 4)):
    case (SCENE_5_TIME + SCENE_5_DURATION + 2 * (SCENE_6_PAUSE / 4)):
    case (SCENE_5_TIME + SCENE_5_DURATION + 3 * (SCENE_6_PAUSE / 4)):
    case SCENE_3_TIME:
      LightCnt = LIGHTNING_UPDATE;
      UpdateValue = 10;
      UpdateCnt = 0;
      ActiveRainCol = ACTIVE_RAIN_1 | ACTIVE_RAIN_2 | ACTIVE_RAIN_3 | ACTIVE_RAIN_4;
      State |= LIGHTNING_1;
      RAIN_TIMER->CR1 |= TIM_CR1_CEN;
    break;
    
    case SCENE_4_TIME:
      ActiveRainCol = ACTIVE_RAIN_1 | ACTIVE_RAIN_2 | ACTIVE_RAIN_3 | ACTIVE_RAIN_4;
      UpdateValue = 10;
      UpdateCnt = 0;
      State |= RAIN;
      RAIN_TIMER->CR1 |= TIM_CR1_CEN;
    break;
    
    case (SCENE_4_TIME +     (SCENE_4_DURATION / 3) - 30):
    case (SCENE_4_TIME + 2 * (SCENE_4_DURATION / 3) - 30):
      LightCnt = LIGHTNING_UPDATE;
      UpdateValue = 10;
      UpdateCnt = 0;
      State |= LIGHTNING_3;
    break;
    
    case (SCENE_5_TIME +     (SCENE_5_DURATION / 6) - 15):
    case (SCENE_5_TIME + 2 * (SCENE_5_DURATION / 6) - 15):
    case (SCENE_5_TIME + 3 * (SCENE_5_DURATION / 6) - 15):
    case (SCENE_5_TIME + 4 * (SCENE_5_DURATION / 6) - 15):
    case (SCENE_5_TIME + 5 * (SCENE_5_DURATION / 6) - 15):
      LightCnt = LIGHTNING_UPDATE;
      UpdateValue = 10;
      UpdateCnt = 0;
      State |= LIGHTNING_2;
    break;
    
    case (SCENE_6_TIME +     (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 2 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 3 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 4 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 5 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 6 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 7 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 8 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME + 9 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME +10 * (SCENE_6_DURATION / 12) - 5):
    case (SCENE_6_TIME +11 * (SCENE_6_DURATION / 12) - 5):
      LightCnt = LIGHTNING_UPDATE;
      UpdateValue = 10;
      UpdateCnt = 0;
      State |= LIGHTNING_1;
    break;
    
    case (SCENE_4_TIME +     (SCENE_4_DURATION / 3)):
    case (SCENE_4_TIME + 2 * (SCENE_4_DURATION / 3)):
    case (SCENE_5_TIME +     (SCENE_5_DURATION / 6)):
    case (SCENE_5_TIME + 2 * (SCENE_5_DURATION / 6)):
    case (SCENE_5_TIME + 3 * (SCENE_5_DURATION / 6)):
    case (SCENE_5_TIME + 4 * (SCENE_5_DURATION / 6)):
    case (SCENE_5_TIME + 5 * (SCENE_5_DURATION / 6)):
    case (SCENE_6_TIME +     (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 2 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 3 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 4 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 5 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 6 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 7 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 8 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME + 9 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME +10 * (SCENE_6_DURATION / 12)):
    case (SCENE_6_TIME +11 * (SCENE_6_DURATION / 12)):
      ThunderDur = 27;
      THUNDER_TIMER->CR1 |= TIM_CR1_CEN;
    break;
    
    case SCENE_5_TIME:
      RainPower = 2;
      UpdateValue = 5;
      UpdateCnt = 0;
      ActiveRainCol = ACTIVE_RAIN_1 | ACTIVE_RAIN_2 | ACTIVE_RAIN_3 | ACTIVE_RAIN_4;
      State |= RAIN;
      RAIN_TIMER->CR1 |= TIM_CR1_CEN;
    break;
    
    case SCENE_6_TIME:
      RainPower = 3;
      UpdateValue = 3;
      UpdateCnt = 0;
      ActiveRainCol = ACTIVE_RAIN_1 | ACTIVE_RAIN_2 | ACTIVE_RAIN_3 | ACTIVE_RAIN_4;
      State |= RAIN;
      RAIN_TIMER->CR1 |= TIM_CR1_CEN;
    break;
    
    default:
    break;
  }
}
//------------------------------------------------------------------------------
void RAIN_TIMER_ISR(void)
{
  uint16_t COLS_PINS[COLS_NUM] = {COL1_PIN, COL2_PIN, COL3_PIN, COL4_PIN};
  uint16_t ROWS_PINS[ROWS_NUM] = {ROW1_PIN, ROW2_PIN, ROW3_PIN, ROW4_PIN, ROW5_PIN};
  
  static uint8_t CurCol = 0;
  static uint8_t LightDur = 0;
  
  uint8_t NewUpdateVal = UpdateValue;
  
  RAIN_TIMER->SR &= ~TIM_SR_UIF;
  
  LEDS_GPIO->ODR &= ~(COLS_MASK | ROWS_MASK);
  
  if ((State & (RAIN | LIGHTNING)) != 0)
  {
  //  
    if ((State & LIGHTNING) != 0)
    {
      if (LightCnt == LIGHTNING_UPDATE)
      {
        if (((CurCol == 0) && ((State & LIGHTNING_1) == LIGHTNING_1))
         || ((CurCol == 2) && ((State & LIGHTNING_2) == LIGHTNING_2))
         || ((CurCol == 3) && ((State & LIGHTNING_3) == LIGHTNING_3)))
        {
          LEDS_GPIO->ODR |= (1 << ROW1_PIN);
        }
        
        else if ((CurCol == 3) && ((State & LIGHTNING_4) == LIGHTNING_4))
        {
          LEDS_GPIO->ODR |= (1 << ROW2_PIN);
        }
      }
    }
    //
    if ((State & RAIN) == RAIN)
    {
      for(uint8_t i = 0; i < RainPower; i++)
      {
        uint8_t LedPosition = CurPos[CurCol] + i;
        
        if (LedPosition == ROWS_NUM)
          break;
        
        LEDS_GPIO->ODR |= 1 << ROWS_PINS[LedPosition];
      }

      if(UpdateCnt == NewUpdateVal)
      {
        uint8_t RAIN_INIT_POS[COLS_NUM] = {RAIN_INIT_POS_COL1, RAIN_INIT_POS_COL2, RAIN_INIT_POS_COL3, RAIN_INIT_POS_COL4};
        
        CurPos[CurCol]++;
        if (CurPos[CurCol] == ROWS_NUM)
          CurPos[CurCol] = RAIN_INIT_POS[CurCol];
      }
    }
  
    if ((ActiveRainCol & (1 << CurCol)) == (1 << CurCol))
      LEDS_GPIO->ODR |= 1 << COLS_PINS[CurCol];
  
    CurCol++;

    if (CurCol == COLS_NUM)
    {
      CurCol = 0;
      UpdateCnt++;
      if (UpdateCnt == (NewUpdateVal + 1))
        UpdateCnt = 0;
      
      if (LightCnt == LIGHTNING_UPDATE)
      {        
        if (LightDur != 0)
          LightDur--;
        else
        {
          if ((State & LIGHTNING_1) == LIGHTNING_1)
          {
            State &= ~LIGHTNING;
            State |= LIGHTNING_2;
          }
          else if ((State & LIGHTNING_2) == LIGHTNING_2)
          {
            State &= ~LIGHTNING;
            State |= LIGHTNING_3;
          }
          else if ((State & LIGHTNING_3) == LIGHTNING_3)
          {
            State &= ~LIGHTNING;
            State |= LIGHTNING_4;
          }
          else if ((State & LIGHTNING_4) == LIGHTNING_4)
          {
            State &= ~LIGHTNING;
            LightCnt++;
          }
          
          LightDur = LIGHTNING_DURATION;
        }
      }         
      else if (LightCnt == (LIGHTNING_UPDATE + 1))
      {
        LightCnt = 0;
      }
      else
        LightCnt++;
    }
  }
}
//------------------------------------------------------------------------------
void THUNDER_TIMER_ISR(void)
{
  THUNDER_TIMER->SR &= ~TIM_SR_UIF;
 
  if (ThunderDur != 0)
  {
    if ((BUZZ_GPIO->ODR & (1 << BUZZ_PIN)) != (1 << BUZZ_PIN))
      BUZZ_GPIO->ODR |= (1 << BUZZ_PIN);
    else
    {
      BUZZ_GPIO->ODR &= ~(1 << BUZZ_PIN);
    
      ThunderDur--;
      if (ThunderDur == 0)
        THUNDER_TIMER->CR1 &= ~TIM_CR1_CEN;
    }
  }
}





